# Copyright (C) 2002,2003  Kazuhiko <kazuhiko@fdiary.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

$KCODE = 'e'

require 'nkf'
require 'eb'
require 'cgi'

LetMeSee_VERSION = '1.0.1'

# for HTML output
class String
  def html_output
    str = CGI.escapeHTML( self )
    str = str.gsub(/\\&lt;/, '<').gsub(/\\&gt;/, '>').gsub(/\\&quot;/, '"')
    str = str.gsub(/<reference>(.+?)<\/reference (.+?)>/, '<a href="./?mode=reference&\2">\1</a>')
    str = str.gsub(/<mono_graphic (.+?)>(.+?)<\/mono_graphic (.+?)>/){"<img src=\"./extract.rb?mode=mono_graphic&#{$3}&#{$1}\">#{$2}</img>"}
    return str
  end
end

# let me see...
class LetMeSee
  attr_reader :dictlist, :dict, :query, :mode, :maxhit, :bookid, :book, :page
  attr_reader :offset, :bookname, :bookpath, :bookdir, :code
  attr_reader :width, :height, :page2, :offset2, :class_book, :section_anchor
  attr_reader :header, :footer

  PATH = File::dirname( __FILE__ )

  def LetMeSee::LetMeSee_VERSION
    LetMeSee_VERSION
  end

  def version
    LetMeSee::LetMeSee_VERSION
  end

  def initialize
    load_conf
    @cgi = CGI::new
    @query = NKF.nkf('-e', @cgi.params['query'][0] || '')
    @dict = @cgi.params['dict']
    @mode = @cgi.params['mode'][0]
    @maxhit = @cgi.params['maxhit'][0].to_i
    @book = @cgi.params['book'][0].to_i
    @page = @cgi.params['page'][0].to_i
    @offset = @cgi.params['offset'][0].to_i
    @page2 = @cgi.params['page2'][0].to_i
    @offset2 = @cgi.params['offset2'][0].to_i
    @code = @cgi.params['code'][0].to_i
    @width = @cgi.params['width'][0].to_i
    @height = @cgi.params['height'][0].to_i
    @bookname = []		# eg. ''
    @bookpath = []		# eg. '/mnt/cdrom'
    @bookdir = []		# eg. 'koujien'
    @bookid = []		# subbook 
    @stoplength = 0x40000
    @dictlist.each do |dictlist|
      b=EB::Book.new
      b.bind(dictlist)
      0.upto(b.subbook_count-1) do |i|
	@bookname.push(b.title(i))
	@bookpath.push(dictlist)
	@bookdir.push(b.directory(i))
	@bookid.push(i)
      end
    end
  end

  def load_conf
    eval( File::open( "#{PATH}/letmesee.conf" ){|f| f.read }.untaint )
    @fontsize = 16 unless @fontsize
    @force_inline = false unless @force_inline
    @fontcode = fontcode(@fontsize)
    @force_inline = false if !@force_inline
    @header = '' if !@header
    @footer = '' if !@footer
    @theme = 'default' if !@theme && !@css
    @class_book = 'book' if !@class_book
    @section_anchor = '' if !@section_anchor
  end

  def theme_url; 'theme'; end

  def css_tag
    if @theme and @theme.length > 0 then
      css = "#{theme_url}/#{@theme}/#{@theme}.css"
    else
      css = @css
    end
    <<-CSS
      <meta http-equiv="content-style-type" content="text/css">
      <link rel="stylesheet" href="#{css}" type="text/css" media="all">
    CSS
  end

  #  (//)
  def search (book)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.hookset = self.hookset(book)
    b.set(self.bookid[book])
    return b.send(mode, query, maxhit ).uniq
  end

  # ե󥹤ɽ
  def content (book, page, offset)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.hookset = self.hookset(book)
    b.set(self.bookid[book])
    return b.content(EB::Position.new(page, offset))
  end

  # ˥塼
  def menu (book)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.hookset = self.hookset(book)
    b.set(self.bookid[book])
    return b.menu if b.menu_available?
  end

  # ɽ
  def copyright (book)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.hookset = self.hookset(book)
    b.set(self.bookid[book])
    return b.copyright if b.copyright_available?
  end

  # ѳν
  def gaiji_w (book, code)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    b.fontcode = @fontcode
    print "Content-Type: image/x-xbitmap\n\n"
    print b.get_widefont(code).to_xbm
  end

  # Ⱦѳν
  def gaiji_n (book, code)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    b.fontcode = @fontcode
    print "Content-Type: image/x-xbitmap\n\n"
    print b.get_narrowfont(code).to_xbm
  end

  # ν
  def mono_graphic (book, page, offset, width, height)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    pos = EB::Position.new(page, offset)
    print "Content-Type: image/bmp\n\n"
    print b.read_monographic(pos, width, height) do |data|
      print data
    end
  end
  def bmp (book, page, offset)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    pos = EB::Position.new(page, offset)
    print "Content-Type: image/bmp\n\n"
    b.read_colorgraphic(pos) do |data|
      print data
    end
  end
  def jpeg (book, page, offset)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    pos = EB::Position.new(page, offset)
    print "Content-Type: image/jpeg\n\n"
    b.read_colorgraphic(pos) do |data|
      print data
    end
  end

  # ν
  def wave (book, page, offset, page2, offset2)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    pos1 = EB::Position.new(page, offset)
    pos2 = EB::Position.new(page2, offset2)
    print "Content-Type: audio/wav\n\n"
    b.read_wavedata(pos1, pos2) do |data|
      print data
    end
  end

  # ưν
  def mpeg (book, page, offset, page2, offset2)
    b = EB::Book.new
    b.bind(self.bookpath[book])
    b.set(self.bookid[book])
    print "Content-Type: video/mpeg\n\n"
    b.read_mpeg(page, offset, page2, offset2) do |data|
      print data
    end
  end

  # Υ
  def fontcode (size)
    case size
      when 16 then return EB::FONT_16
      when 24 then return EB::FONT_24
      when 30 then return EB::FONT_30
      when 48 then return EB::FONT_48
      else @fontsize = 16; return EB::FONT_16
    end
  end

  # Ƽեå
  def hookset (book)
    h = EB::Hookset.new
    h.register(EB::HOOK_NEWLINE) do |eb2,argv|
      "\\<br\\>\n"
    end
    h.register(EB::HOOK_WIDE_FONT) do |eb2,argv|
      format('\<img src=\"extract.rb?book=%d&mode=gaiji_w&code=%d\" alt=\"_\" width=\"%d\" height=\"%d\"\>',book, argv[0], @fontsize, @fontsize)
    end
    h.register(EB::HOOK_NARROW_FONT) do |eb2,argv|
      format('\<img src=\"extract.rb?book=%d&mode=gaiji_n&code=%d\" alt=\"_\" width=\"%d\" height=\"%d\"\>',book, argv[0], @fontsize / 2, @fontsize)
    end
    h.register(EB::HOOK_BEGIN_EMPHASIS) do |eb2,argv|
      '\<strong\>'
    end
    h.register(EB::HOOK_END_EMPHASIS) do |eb2,argv|
      '\</strong\>'
    end
    h.register(EB::HOOK_BEGIN_SUBSCRIPT) do |eb2,argv|
      '\<sub\>'
    end
    h.register(EB::HOOK_END_SUBSCRIPT) do |eb2,argv|
      '\</sub\>'
    end
    h.register(EB::HOOK_BEGIN_SUPERSCRIPT) do |eb2,argv|
      '\<sup\>'
    end
    h.register(EB::HOOK_END_SUPERSCRIPT) do |eb2,argv|
      '\</sup\>'
    end
    h.register(EB::HOOK_BEGIN_REFERENCE) do |eb2,argv|
      '\<span class=\"reference\"\>\<reference\>'
    end
    h.register(EB::HOOK_END_REFERENCE) do |eb2,argv|
      dictlist = ''
      self.dict.each do |dict|
	dictlist += "&dict=#{dict.to_i}"
      end
      format('\</reference book=%d&page=%d&offset=%d%s\>\</span\>',book, argv[1], argv[2], dictlist)
    end
    h.register(EB::HOOK_BEGIN_CANDIDATE) do |eb2,argv|
      '\<span class=\"reference\"\>\<reference\>'
    end
    h.register(EB::HOOK_END_CANDIDATE_GROUP) do |eb2,argv|
      dictlist = ''
      self.dict.each do |dict|
	dictlist += "&dict=#{dict.to_i}"
      end
      format('\</reference book=%d&page=%d&offset=%d%s\>\</span\>',book, argv[1], argv[2], dictlist)
    end
    h.register(EB::HOOK_BEGIN_MONO_GRAPHIC) do |eb2,argv|
      format('\<mono_graphic width=%d&height=%d\>', argv[3], argv[2])
    end
    h.register(EB::HOOK_END_MONO_GRAPHIC) do |eb2,argv|
      format('\</mono_graphic book=%d&page=%d&offset=%d\>',book, argv[1], argv[2])
    end
    h.register(EB::HOOK_BEGIN_COLOR_BMP) do |eb2,argv|
      if @force_inline
	format('\<img src=\"extract.rb?mode=bmp&book=%d&page=%d&offset=%d\" alt=\"[]\"\>',book, argv[2], argv[3])
      else
	format('\<a href=\"extract.rb?mode=bmp&book=%d&page=%d&offset=%d\"\>[] ',book, argv[2], argv[3])
      end
    end
    h.register(EB::HOOK_BEGIN_COLOR_JPEG) do |eb2,argv|
      if @force_inline
	format('\<img src=\"extract.rb?mode=jpeg&book=%d&page=%d&offset=%d\" alt=\"[]\"\>',book, argv[2], argv[3])
      else
	format('\<a href=\"extract.rb?mode=jpeg&book=%d&page=%d&offset=%d\"\>[] ',book, argv[2], argv[3])
      end
    end
    h.register(EB::HOOK_END_COLOR_GRAPHIC) do |eb2,argv|
      '\</a\>'
    end
    h.register(EB::HOOK_BEGIN_IN_COLOR_BMP) do |eb2,argv|
      format('\<img src=\"extract.rb?mode=bmp&book=%d&page=%d&offset=%d\" alt=\"[]\"\>',book, argv[2], argv[3])
    end if EB.const_defined?(:HOOK_BEGIN_IN_COLOR_BMP)
    h.register(EB::HOOK_BEGIN_IN_COLOR_JPEG) do |eb2,argv|
      format('\<img src=\"extract.rb?mode=jpeg&book=%d&page=%d&offset=%d\" alt=\"[]\"\>',book, argv[2], argv[3])
    end if EB.const_defined?(:HOOK_BEGIN_IN_COLOR_JPEG)
    h.register(EB::HOOK_END_IN_COLOR_GRAPHIC) do |eb2,argv|
      '\</a\>'
    end
    h.register(EB::HOOK_BEGIN_WAVE) do |eb2,argv|
      format('\<a href=\"extract.rb?mode=wave&book=%d&page=%d&offset=%d&page2=%d&offset2=%d\"\>[] ',book, argv[2], argv[3], argv[4], argv[5])
    end
    h.register(EB::HOOK_END_WAVE) do |eb2,argv|
      '\</a\>'
    end
    h.register(EB::HOOK_BEGIN_MPEG) do |eb2,argv|
      format('\<a href=\"extract.rb?mode=mpeg&book=%d&page=%d&offset=%d&page2=%d&offset2=%d\"\>[ư] ',book, argv[2], argv[3], argv[4], argv[5])
    end
    h.register(EB::HOOK_END_MPEG) do |eb2,argv|
      '\</a\>'
    end
    return h
  end
end
